# type-k-seconds


# my_skaters = top_rated_models(k=1, n=15, max_seconds=1, category='univariate')
U_1_1 = ['slow_precision_ema_ensemble', 'quick_precision_ema_ensemble', 'slow_aggressive_ema_ensemble', 'balanced_ema_ensemble', 'precision_ema_ensemble', 'quick_balanced_ema_ensemble', 'quick_aggressive_ema_ensemble', 'slow_balanced_ema_ensemble', 'sk_theta', 'aggressive_ema_ensemble', 'thinking_fast_and_fast', 'thinking_precision_ensemble', 'thinking_fast_and_slow', 'quickly_moving_average', 'slowly_moving_average']
#  my_skaters = top_rated_models(k=1, n=15, max_seconds=5,category='univariate')
U_1_5 = ['slow_precision_ema_ensemble', 'quick_precision_ema_ensemble', 'slow_aggressive_ema_ensemble', 'balanced_ema_ensemble', 'precision_ema_ensemble', 'tsa_aggressive_theta_ensemble', 'quick_balanced_ema_ensemble', 'tsa_balanced_theta_ensemble', 'quick_aggressive_ema_ensemble', 'slow_balanced_ema_ensemble', 'sk_theta', 'aggressive_ema_ensemble', 'tsa_precision_theta_ensemble', 'thinking_fast_and_fast', 'thinking_precision_ensemble']

U_3_1 = ['quick_aggressive_ema_ensemble', 'slow_precision_ema_ensemble', 'sluggish_moving_average', 'precision_ema_ensemble', 'aggressive_ema_ensemble', 'thinking_slow_and_fast', 'sk_theta', 'slow_aggressive_ema_ensemble', 'quickly_moving_average', 'balanced_ema_ensemble', 'thinking_slow_and_slow', 'quick_precision_ema_ensemble', 'slow_balanced_ema_ensemble', 'quick_balanced_ema_ensemble', 'slowly_moving_average']
U_8_1 = ['slow_precision_ema_ensemble', 'quick_precision_ema_ensemble', 'sluggish_moving_average', 'precision_ema_ensemble', 'thinking_slow_and_slow', 'slow_balanced_ema_ensemble', 'slow_aggressive_ema_ensemble', 'thinking_slow_and_fast', 'balanced_ema_ensemble', 'slowly_moving_average', 'aggressive_ema_ensemble', 'sk_theta', 'quickly_moving_average', 'quick_aggressive_ema_ensemble', 'thinking_fast_and_slow']
U_21_1 = ['thinking_slow_and_slow', 'sluggish_moving_average', 'slowly_moving_average', 'thinking_precision_ensemble', 'thinking_fast_and_slow', 'precision_ema_ensemble', 'thinking_slow_and_fast', 'quick_aggressive_ema_ensemble', 'slow_precision_ema_ensemble', 'pmd_univariate', 'quick_balanced_ema_ensemble', 'balanced_ema_ensemble', 'slow_balanced_ema_ensemble', 'quick_precision_ema_ensemble', 'aggressive_ema_ensemble']





#  my_skaters = top_rated_models(k=1, n=15, max_seconds=1,category='residual')
R_1_1 = ['quick_precision_ema_ensemble', 'thinking_fast_and_fast', 'precision_ema_ensemble', 'slow_precision_ema_ensemble', 'aggressive_ema_ensemble', 'balanced_ema_ensemble', 'slow_aggressive_ema_ensemble', 'slow_balanced_ema_ensemble', 'thinking_precision_ensemble', 'rapidly_moving_average', 'quick_aggressive_ema_ensemble', 'quickly_moving_average', 'quick_balanced_ema_ensemble', 'sk_theta', 'thinking_fast_and_slow']
# my_skaters = top_rated_models(k=1, n=15, max_seconds=5,category='residual')
R_1_5 = ['quick_precision_ema_ensemble', 'thinking_fast_and_fast', 'precision_ema_ensemble', 'tsa_precision_theta_ensemble', 'slow_precision_ema_ensemble', 'aggressive_ema_ensemble', 'balanced_ema_ensemble', 'slow_aggressive_ema_ensemble', 'tsa_balanced_theta_ensemble', 'slow_balanced_ema_ensemble', 'thinking_precision_ensemble', 'tsa_aggressive_theta_ensemble', 'rapidly_moving_average', 'quick_aggressive_ema_ensemble', 'quickly_moving_average']



